@@ -19,7 +19,7 @@
 #include <math.h>
 #include <numpy/arrayobject.h>
 
-#include "imageutil.h"
+#include "include/imageutil.h"
 
 //#undef __ARM_NEON__
 
@@ -56,11 +56,11 @@ static PyObject *ScannerError;
 #define MIN(a,b) ((a)<(b)?(a):(b))
 #define MAX(a,b) ((a)>(b)?(a):(b))
 
-#define MAX_REGIONS 200
+#define MAX_REGIONS 2000
 
 struct scan_params {
-    uint16_t min_region_size;
-    uint16_t max_region_size;
+    uint16_t min_region_area;
+    uint16_t max_region_area;
     uint16_t min_region_size_xy;
     uint16_t max_region_size_xy;
     uint16_t histogram_count_threshold;
@@ -69,8 +69,8 @@ struct scan_params {
 };
 
 static const struct scan_params scan_params_640_480 = {
-	min_region_size : 8,
-        max_region_size : 400,
+	min_region_area : 8,
+        max_region_area : 400,
         min_region_size_xy : 2,
         max_region_size_xy : 30,
         histogram_count_threshold : 50,
@@ -94,6 +94,24 @@ struct regions {
         int16_t **data;
 };
 
+#define SHOW_TIMING 0
+#if SHOW_TIMING
+struct timeval tp1,tp2;
+
+static void start_timer()
+{
+	gettimeofday(&tp1,NULL);
+}
+
+static double end_timer()
+{
+	gettimeofday(&tp2,NULL);
+	return((tp2.tv_sec - tp1.tv_sec) + 
+	       (tp2.tv_usec - tp1.tv_usec)*1.0e-6);
+}
+#endif // SHOW_TIMING
+
+
 /*
   save a bgr image as a P6 pnm file
  */
@@ -276,11 +294,20 @@ static void NOINLINE get_min_max_neon(const struct bgr * __restrict in,
 
 	min->r = min->g = min->b = 255;
 	max->r = max->g = max->b = 0;
+	/*
+	  we split this into 3 parts as gcc 4.8.1 on ARM runs out of
+	  registers and gives a spurious const error if we leave it as
+	  one chunk
+	 */
 	for (i=0; i<8; i++) {
 		if (min->b > vget_lane_u8(bmin, i)) min->b = vget_lane_u8(bmin, i);
 		if (min->g > vget_lane_u8(gmin, i)) min->g = vget_lane_u8(gmin, i);
+	}
+	for (i=0; i<8; i++) {
 		if (min->r > vget_lane_u8(rmin, i)) min->r = vget_lane_u8(rmin, i);
 		if (max->b < vget_lane_u8(bmax, i)) max->b = vget_lane_u8(bmax, i);
+	}
+	for (i=0; i<8; i++) {
 		if (max->g < vget_lane_u8(gmax, i)) max->g = vget_lane_u8(gmax, i);
 		if (max->r < vget_lane_u8(rmax, i)) max->r = vget_lane_u8(rmax, i);
 	}
@@ -580,7 +607,7 @@ static unsigned find_region(const struct scan_params *scan_params,
 			    int y, int x)
 {
 	int yofs, xofs;
-        uint16_t m = scan_params->region_merge;
+        uint16_t m = MAX(1, scan_params->region_merge/10);
 
 	/*
 	  we only need to look up or directly to the left, as this function is used
@@ -680,12 +707,18 @@ static void remove_region(struct regions *in, unsigned i)
         in->num_regions--;
 }
 
-static bool regions_overlap(const struct region_bounds *r1, const struct region_bounds *r2)
+/*
+  determine if two regions overlap, taking into account the
+  region_merge size
+ */
+static bool regions_overlap(const struct scan_params *scan_params,
+                            const struct region_bounds *r1, const struct region_bounds *r2)
 {
-    if (r1->maxx < r2->minx) return false;
-    if (r2->maxx < r1->minx) return false;
-    if (r1->maxy < r2->miny) return false;
-    if (r2->maxy < r1->miny) return false;
+    uint16_t m = scan_params->region_merge;
+    if (r1->maxx+m < r2->minx) return false;
+    if (r2->maxx+m < r1->minx) return false;
+    if (r1->maxy+m < r2->miny) return false;
+    if (r2->maxy+m < r1->miny) return false;
     return true;
 }
 
@@ -700,7 +733,7 @@ static void merge_regions(const struct scan_params *scan_params, struct regions
                 found_overlapping = false;
                 for (i=0; i<in->num_regions; i++) {
                         for (j=i+1; j<in->num_regions; j++) {
-                                if (regions_overlap(&in->bounds[i], &in->bounds[j])) {
+                            if (regions_overlap(scan_params, &in->bounds[i], &in->bounds[j])) {
                                         found_overlapping = true;
                                         struct region_bounds *b1 = &in->bounds[i];                                        
                                         struct region_bounds *b2 = &in->bounds[j];                                        
@@ -717,25 +750,22 @@ static void merge_regions(const struct scan_params *scan_params, struct regions
 }
 
 /*
-  remove any too small or large regions
+  remove any too large regions
  */
-static void prune_regions(const struct scan_params *scan_params, struct regions *in)
+static void prune_large_regions(const struct scan_params *scan_params, struct regions *in)
 {
 	unsigned i;
 	for (i=0; i<in->num_regions; i++) {
-		if (in->region_size[i] < scan_params->min_region_size ||
-		    in->region_size[i] > scan_params->max_region_size ||
+		if (in->region_size[i] > scan_params->max_region_area ||
 		    (in->bounds[i].maxx - in->bounds[i].minx) > scan_params->max_region_size_xy ||
-		    (in->bounds[i].maxx - in->bounds[i].minx) < scan_params->min_region_size_xy ||
-		    (in->bounds[i].maxy - in->bounds[i].miny) > scan_params->max_region_size_xy ||
-		    (in->bounds[i].maxy - in->bounds[i].miny) < scan_params->min_region_size_xy) {
+		    (in->bounds[i].maxy - in->bounds[i].miny) > scan_params->max_region_size_xy) {
 #if 0
                         printf("prune size=%u xsize=%u ysize=%u range=(min:%u,max:%u,minxy:%u,maxxy:%u)\n",
                                in->region_size[i], 
                                in->bounds[i].maxx - in->bounds[i].minx,
                                in->bounds[i].maxy - in->bounds[i].miny,
-                               scan_params->min_region_size, 
-                               scan_params->max_region_size,
+                               scan_params->min_region_area, 
+                               scan_params->max_region_area,
                                scan_params->min_region_size_xy, 
                                scan_params->max_region_size_xy);
 #endif
@@ -746,6 +776,34 @@ static void prune_regions(const struct scan_params *scan_params, struct regions
 	}
 }
 
+/*
+  remove any too small regions
+ */
+static void prune_small_regions(const struct scan_params *scan_params, struct regions *in)
+{
+	unsigned i;
+	for (i=0; i<in->num_regions; i++) {
+		if (in->region_size[i] < scan_params->min_region_area ||
+		    (in->bounds[i].maxx - in->bounds[i].minx) < scan_params->min_region_size_xy ||
+		    (in->bounds[i].maxy - in->bounds[i].miny) < scan_params->min_region_size_xy) {
+#if 0
+                        printf("prune size=%u xsize=%u ysize=%u range=(min:%u,max:%u,minxy:%u,maxxy:%u)\n",
+                               in->region_size[i], 
+                               in->bounds[i].maxx - in->bounds[i].minx,
+                               in->bounds[i].maxy - in->bounds[i].miny,
+                               scan_params->min_region_area, 
+                               scan_params->max_region_area,
+                               scan_params->min_region_size_xy, 
+                               scan_params->max_region_size_xy);
+#endif
+                        remove_region(in, i);
+			i--;
+		}
+		    
+	}
+}
+
+
 /*
   score one region in an image
 
@@ -986,8 +1044,8 @@ static void scale_scan_params(struct scan_params *scan_params, uint32_t height,
     if (ascale < 1.0) ascale = 1.0;
     if (wscale < 1.0) wscale = 1.0;
     *scan_params = scan_params_640_480;
-    scan_params->min_region_size *= ascale;
-    scan_params->max_region_size *= ascale;
+    scan_params->min_region_area *= ascale;
+    scan_params->max_region_area *= ascale;
     scan_params->min_region_size_xy *= wscale;
     scan_params->max_region_size_xy *= wscale;
     scan_params->histogram_count_threshold *= ascale;
@@ -1015,12 +1073,24 @@ static void scale_scan_params_user(struct scan_params *scan_params, uint32_t hei
     float meters_per_pixel = dict_lookup(parm_dict, "MetersPerPixel", 0.25);
     float meters_per_pixel2 = meters_per_pixel * meters_per_pixel;
     *scan_params = scan_params_640_480;
-    scan_params->min_region_size = MAX(dict_lookup(parm_dict, "MinRegionArea", 1.0) / meters_per_pixel2, 1);
-    scan_params->max_region_size = MAX(dict_lookup(parm_dict, "MaxRegionArea", 4.0) / meters_per_pixel2, 1);
+    scan_params->min_region_area = MAX(dict_lookup(parm_dict, "MinRegionArea", 1.0) / meters_per_pixel2, 1);
+    scan_params->max_region_area = MAX(dict_lookup(parm_dict, "MaxRegionArea", 4.0) / meters_per_pixel2, 1);
     scan_params->min_region_size_xy = MAX(dict_lookup(parm_dict, "MinRegionSize", 0.25) / meters_per_pixel, 1);
     scan_params->max_region_size_xy = MAX(dict_lookup(parm_dict, "MaxRegionSize", 4.0) / meters_per_pixel, 1);
     scan_params->histogram_count_threshold = MAX(dict_lookup(parm_dict, "MaxRarityPct", 0.016) * (width*height)/100.0, 1);
     scan_params->region_merge = MAX(dict_lookup(parm_dict, "RegionMergeSize", 0.5) / meters_per_pixel, 1);
+    scan_params->save_intermediate = dict_lookup(parm_dict, "SaveIntermediate", 0);
+    if (scan_params->save_intermediate) {
+        printf("mpp=%f mpp2=%f min_region_area=%u max_region_area=%u min_region_size_xy=%u max_region_size_xy=%u histogram_count_threshold=%u region_merge=%u\n",
+               meters_per_pixel,
+               meters_per_pixel2,
+               scan_params->min_region_area,
+               scan_params->max_region_area,
+               scan_params->min_region_size_xy,
+               scan_params->max_region_size_xy,
+               scan_params->histogram_count_threshold,
+               scan_params->region_merge);
+    }
 }
 
 /*
@@ -1033,6 +1103,10 @@ scanner_scan(PyObject *self, PyObject *args)
 	PyArrayObject *img_in;
         PyObject *parm_dict = NULL;
 
+#if SHOW_TIMING
+        start_timer();
+#endif
+        
 	if (!PyArg_ParseTuple(args, "O|O", &img_in, &parm_dict))
 		return NULL;
 
@@ -1090,8 +1164,9 @@ scanner_scan(PyObject *self, PyObject *args)
                 free(marked);
         }
 
+	prune_large_regions(&scan_params, regions);
 	merge_regions(&scan_params, regions);
-	prune_regions(&scan_params, regions);
+	prune_small_regions(&scan_params, regions);
 
         if (scan_params.save_intermediate) {
                 struct bgr_image *marked;
@@ -1126,6 +1201,10 @@ scanner_scan(PyObject *self, PyObject *args)
 
 	free(regions);
 
+#if SHOW_TIMING
+        printf("dt=%f\n", end_timer());
+#endif
+
 	return list;
 }
 
